using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Syncfusion.Grouping;
using Syncfusion.Windows.Forms.Grid.Grouping;
using Syncfusion.Windows.Forms.Grid;

namespace GGCSort
{
    public partial class Form1 : Form
    {
        private const string format = "###,###";
        private ListSortDirection tvSort = ListSortDirection.Descending;
        private ListSortDirection notionalSort = ListSortDirection.Descending;

        public Form1()
        {
            InitializeComponent();
            OptimizeGrid();
            LoadData();
            SetUpGrouping();
        }

        private void OptimizeGrid()
        {
            // Optimize memory footprint, drawing and ListChanged performance

            // Use less memory for internal binary tree structures
            gridGroupingControl1.CounterLogic = EngineCounters.YAmount;
            gridGroupingControl1.AllowedOptimizations = EngineOptimizations.DisableCounters | EngineOptimizations.RecordsAsDisplayElements;

            // Affected settings by UseDefaultsForFasterDrawing are: InvalidateAllWhenListChanged, 
            // TableOptions.ColumnsMaxLengthStrategy, TableOptions.GridLineBorder, 
            // TableOptions.DrawTextWithGdiInterop, TableOptions.VerticalPixelScroll,
            // Appearance.AnyRecordFieldCell.WrapText and Appearance.AnyRecordFieldCell.Trimming.
            gridGroupingControl1.UseDefaultsForFasterDrawing = true;

            // Instruct grouping control not to access datasource through
            // CurrencyManager API and instead access list diretly. When set to
            // false you wil however loose support for built-in CurrencyManager 
            // synchronization, so only do this if you don't need CurrencyManager synchronization.
            gridGroupingControl1.BindToCurrencyManager = false;

            // Update display with changes, optionally use ScrollWindow for insert and removing rows.
            //gridGroupingControl1.UpdateDisplayFrequency = 0;  // only manual updates (when you call grid.Update)
            gridGroupingControl1.UpdateDisplayFrequency = 1;  // immediate update after each ListChanged.
            //gridGroupingControl1.UpdateDisplayFrequency = 100;  // update every 100 ms.
            gridGroupingControl1.InsertRemoveBehavior = GridListChangedInsertRemoveBehavior.ScrollWithImmediateUpdate;  // Scrollwindow will cause immediate update.
            gridGroupingControl1.SortPositionChangedBehavior = GridListChangedInsertRemoveBehavior.ScrollWithImmediateUpdate;  // Scrollwindow will cause immediate update.

            // Don't forget this property. If it is true then InsertRemoveBehavior and SortPositionChangedBehavior
            // properties would have no effect at all. Its default value is false if UseDefaultsForFasterDrawing = true
            // has been set, but setting it explictly makes sure this is false.
            gridGroupingControl1.InvalidateAllWhenListChanged = false;

            //Reduce flickering
            //this.gridGroupingControl1.TableControl.EnableDoubleBufferSurface();
        }

        private void LoadData()
        {
            this.gridGroupingControl1.DataSource = GetCDSDT();
        }

        private void SetUpGrouping()
        {

            this.gridGroupingControl1.Appearance.AnyCell.ReadOnly = true;

            // Do not show group drop area
            this.gridGroupingControl1.ShowGroupDropArea = false;
            this.gridGroupingControl1.ShowNavigationBar = false;
            this.gridGroupingControl1.ThemesEnabled = true;
            this.gridGroupingControl1.TableOptions.AllowSortColumns = true;

            // Hide row headers
            this.gridGroupingControl1.TableOptions.ShowRowHeader = false;

            //Freeze the pane at 2 rows (Column Header and Totals Summary
            this.gridGroupingControl1.TableModel.Rows.FrozenCount = 2;
            this.gridGroupingControl1.TableModel.Cols.FrozenCount = 1;

            // Set up the event to handlers
            this.gridGroupingControl1.QueryCellStyleInfo += new GridTableCellStyleInfoEventHandler(gridGroupingControl1_QueryCellStyleInfo);            
            this.gridGroupingControl1.TableControlQueryAllowSortColumn += new GridQueryAllowSortColumnEventHandler(gridGroupingControl1_TableControlQueryAllowSortColumn);            
            
            // Specify the grouping level
            this.gridGroupingControl1.TableDescriptor.GroupedColumns.Add("Region");
            this.gridGroupingControl1.TableDescriptor.GroupedColumns.Add("Issuer");
            this.gridGroupingControl1.TableDescriptor.GroupedColumns.Add("Book");

            // Set up top group look
            this.gridGroupingControl1.TopLevelGroupOptions.CaptionText = "";
            this.gridGroupingControl1.TopLevelGroupOptions.ShowAddNewRecordAfterDetails = false;
            this.gridGroupingControl1.TopLevelGroupOptions.ShowAddNewRecordBeforeDetails = false;
            this.gridGroupingControl1.TopLevelGroupOptions.ShowCaptionSummaryCells = true;
            this.gridGroupingControl1.TopLevelGroupOptions.ShowCaption = true;
            this.gridGroupingControl1.TopLevelGroupOptions.CaptionSummaryRow = "Totals";
            this.gridGroupingControl1.TopLevelGroupOptions.ShowSummaries = false;

            // Set up child group look
            this.gridGroupingControl1.ChildGroupOptions.CaptionText = "{Category}";
            this.gridGroupingControl1.ChildGroupOptions.ShowAddNewRecordAfterDetails = false;
            this.gridGroupingControl1.ChildGroupOptions.ShowAddNewRecordBeforeDetails = false;
            this.gridGroupingControl1.ChildGroupOptions.ShowCaptionSummaryCells = true;
            this.gridGroupingControl1.ChildGroupOptions.ShowCaption = true;
            this.gridGroupingControl1.ChildGroupOptions.CaptionSummaryRow = "Totals";
            this.gridGroupingControl1.ChildGroupOptions.ShowSummaries = false;
                 

            // Specify the summary data
            GridSummaryColumnDescriptor sumTV = new GridSummaryColumnDescriptor();
            sumTV.SummaryType = SummaryType.DoubleAggregate;
            sumTV.DataMember = "Notional";
            sumTV.DisplayColumn = "Notional";
            sumTV.Format = "{Sum:###,###}";
            sumTV.Name = "SUMMARY_NOTIONAL";


            GridSummaryColumnDescriptor sumDelta = new GridSummaryColumnDescriptor();
            sumDelta.SummaryType = SummaryType.DoubleAggregate;
            sumDelta.DataMember = "TV";
            sumDelta.DisplayColumn = "TV";
            sumDelta.Format = "{Sum:###,###}";
            sumDelta.Name = "SUMMARY_TV";          

            // Create a summary row and add the summary col to it
            GridSummaryRowDescriptor sumRow = new GridSummaryRowDescriptor("Totals", "TOTALS", new GridSummaryColumnDescriptor[] { sumTV, sumDelta });

            //Add the summary row to the table descriptor
            this.gridGroupingControl1.TableDescriptor.SummaryRows.Add(sumRow);

            // Set up GroupCaptionCell
            gridGroupingControl1.TableDescriptor.Appearance.GroupCaptionCell.CellType = "Static";
            gridGroupingControl1.TableDescriptor.Appearance.GroupCaptionCell.Themed = false;            
            gridGroupingControl1.TableDescriptor.Appearance.GroupCaptionCell.Borders.All = new GridBorder(GridBorderStyle.Standard, Color.Gray, GridBorderWeight.ExtraThin);
            gridGroupingControl1.TableDescriptor.Appearance.GroupCaptionCell.CellAppearance = GridCellAppearance.Flat;
            gridGroupingControl1.TableDescriptor.Appearance.GroupCaptionCell.BackColor = Color.Bisque;
            gridGroupingControl1.TableDescriptor.Appearance.GroupCaptionCell.Font.Bold = true;

            // Set up ColHeader                
            this.gridGroupingControl1.TableDescriptor.Appearance.ColumnHeaderCell.CellType = "ColumnHeaderCell";            
            gridGroupingControl1.TableDescriptor.Appearance.ColumnHeaderCell.Borders.All = GridBorder.Empty;
            gridGroupingControl1.TableDescriptor.Appearance.ColumnHeaderCell.Borders.All = new GridBorder(GridBorderStyle.Standard, Color.Gray, GridBorderWeight.ExtraThin);
            gridGroupingControl1.TableDescriptor.Appearance.ColumnHeaderCell.CellAppearance = GridCellAppearance.Flat;
            gridGroupingControl1.TableDescriptor.Appearance.ColumnHeaderCell.BackColor = Color.Bisque;
            gridGroupingControl1.TableDescriptor.Appearance.ColumnHeaderCell.Font.Bold = true;
            gridGroupingControl1.TableDescriptor.Appearance.ColumnHeaderCell.HorizontalAlignment = GridHorizontalAlignment.Center;

            // Set up GroupCaptionSummary cell        
            gridGroupingControl1.TableDescriptor.Appearance.GroupCaptionSummaryCell.CellType = "Static";
            gridGroupingControl1.TableDescriptor.Appearance.GroupCaptionSummaryCell.Themed = false;            
            gridGroupingControl1.TableDescriptor.Appearance.GroupCaptionSummaryCell.Borders.All = new GridBorder(GridBorderStyle.Standard, Color.Gray, GridBorderWeight.ExtraThin);
            gridGroupingControl1.TableDescriptor.Appearance.GroupCaptionSummaryCell.CellAppearance = GridCellAppearance.Flat;
            gridGroupingControl1.TableDescriptor.Appearance.GroupCaptionSummaryCell.BackColor = Color.Tan;
            gridGroupingControl1.TableDescriptor.Appearance.GroupCaptionSummaryCell.Font.Bold = true;
            gridGroupingControl1.TableDescriptor.Appearance.GroupCaptionSummaryCell.HorizontalAlignment = GridHorizontalAlignment.Right;
            gridGroupingControl1.TableDescriptor.Appearance.GroupCaptionSummaryCell.Format = format;


            // Set up alternate rows
            gridGroupingControl1.TableDescriptor.Appearance.RecordFieldCell.CellType = "Static";
            gridGroupingControl1.TableDescriptor.Appearance.RecordFieldCell.BackColor = Color.Lavender;
            gridGroupingControl1.TableDescriptor.Appearance.RecordFieldCell.Borders.All = new GridBorder(GridBorderStyle.Standard, Color.Gray, GridBorderWeight.ExtraThin);
            gridGroupingControl1.TableDescriptor.Appearance.RecordFieldCell.HorizontalAlignment = GridHorizontalAlignment.Right;
            gridGroupingControl1.TableDescriptor.Appearance.RecordFieldCell.Format = format;
            gridGroupingControl1.TableDescriptor.Appearance.AlternateRecordFieldCell.CellType = "Static";
            gridGroupingControl1.TableDescriptor.Appearance.AlternateRecordFieldCell.BackColor = Color.LightGray;
            gridGroupingControl1.TableDescriptor.Appearance.AlternateRecordFieldCell.HorizontalAlignment = GridHorizontalAlignment.Right;
            gridGroupingControl1.TableDescriptor.Appearance.AlternateRecordFieldCell.Borders.All = new GridBorder(GridBorderStyle.Standard, Color.Gray, GridBorderWeight.ExtraThin);
            gridGroupingControl1.TableDescriptor.Appearance.AlternateRecordFieldCell.Format = format;

        }

        void gridGroupingControl1_QueryCellStyleInfo(object sender, GridTableCellStyleInfoEventArgs e)
        {
            if (e.TableCellIdentity.TableCellType == GridTableCellType.GroupCaptionSummaryCell)
            {
                if (e.TableCellIdentity.RowIndex == 1 && e.TableCellIdentity.ColIndex == 0)
                    e.Style.CellValue = "TOTALS";
            }            
            e.Handled = true;
        }

        void gridGroupingControl1_TableControlQueryAllowSortColumn(object sender, GridQueryAllowSortColumnEventArgs e)
        {    

            if (this.gridGroupingControl1.TableDescriptor.GroupedColumns.Count > 0)
            {

                GridSummaryRowDescriptor summaryRow = this.gridGroupingControl1.TableDescriptor.SummaryRows["Totals"];
                SortColumnDescriptor gsd = this.gridGroupingControl1.TableDescriptor.GroupedColumns[0];
                SortColumnDescriptor gsd1 = this.gridGroupingControl1.TableDescriptor.GroupedColumns[1];
                SortColumnDescriptor gsd2 = this.gridGroupingControl1.TableDescriptor.GroupedColumns[2];
                                            
                if (e.Column.Name.Equals("Notional", StringComparison.InvariantCultureIgnoreCase))
                {
                    GridSummaryColumnDescriptor summary = summaryRow.SummaryColumns["SUMMARY_NOTIONAL"];
                    if (notionalSort == ListSortDirection.Ascending)
                    {
                        notionalSort = ListSortDirection.Descending;
                        gsd.SetGroupSummarySortOrder(summary.GetSummaryDescriptorName(), "Sum", notionalSort);
                        gsd1.SetGroupSummarySortOrder(summary.GetSummaryDescriptorName(), "Sum", notionalSort);
                        gsd2.SetGroupSummarySortOrder(summary.GetSummaryDescriptorName(), "Sum", notionalSort);
                    }
                    else
                    {
                        notionalSort = ListSortDirection.Ascending;
                        gsd.SetGroupSummarySortOrder(summary.GetSummaryDescriptorName(), "Sum", notionalSort);
                        gsd1.SetGroupSummarySortOrder(summary.GetSummaryDescriptorName(), "Sum", notionalSort);
                        gsd2.SetGroupSummarySortOrder(summary.GetSummaryDescriptorName(), "Sum", notionalSort);
                    }
                }
                else if (e.Column.Name.Equals("TV", StringComparison.InvariantCultureIgnoreCase))
                {
                    GridSummaryColumnDescriptor summary = summaryRow.SummaryColumns["SUMMARY_TV"];
                    if (tvSort == ListSortDirection.Ascending)
                    {
                        tvSort = ListSortDirection.Descending;
                        gsd.SetGroupSummarySortOrder(summary.GetSummaryDescriptorName(), "Sum", tvSort);
                        gsd1.SetGroupSummarySortOrder(summary.GetSummaryDescriptorName(), "Sum", tvSort);
                        gsd2.SetGroupSummarySortOrder(summary.GetSummaryDescriptorName(), "Sum", tvSort);
                    }
                    else
                    {
                        tvSort = ListSortDirection.Ascending;
                        gsd.SetGroupSummarySortOrder(summary.GetSummaryDescriptorName(), "Sum", tvSort);
                        gsd1.SetGroupSummarySortOrder(summary.GetSummaryDescriptorName(), "Sum", tvSort);
                        gsd2.SetGroupSummarySortOrder(summary.GetSummaryDescriptorName(), "Sum", tvSort);
                    }
                }               
            }

        }
        
        private DataTable GetCDSDT()
        {
            DataTable dt = new DataTable("CDS");
            dt.Columns.Add(new DataColumn("Issuer", typeof(string)));
            dt.Columns.Add(new DataColumn("BondName", typeof(string)));
            dt.Columns.Add(new DataColumn("Rating", typeof(string)));
            dt.Columns.Add(new DataColumn("ProductType", typeof(string)));
            dt.Columns.Add(new DataColumn("Book", typeof(string)));
            dt.Columns.Add(new DataColumn("Trader", typeof(string)));
            dt.Columns.Add(new DataColumn("Region", typeof(string)));
            dt.Columns.Add(new DataColumn("Notional", typeof(double)));
            dt.Columns.Add(new DataColumn("TV", typeof(double)));
            dt.Columns.Add(new DataColumn("Delta", typeof(double)));
            dt.Columns.Add(new DataColumn("Stress", typeof(double)));

            dt.Rows.Add(new object[] { "ABN Amro", "Bond-ABC", "AAA", "CDS", "NY100", "Trader-1", "NORTH AMERICA", 1000, -100, -100, -100 });
            dt.Rows.Add(new object[] { "ABN Amro", "Bond-DEF", "AAA", "CDS", "NY100", "Trader-2", "NORTH AMERICA", 1000, -100, -100, -100 });
            dt.Rows.Add(new object[] { "ABN Amro", "Bond-GHI", "AA+", "CDS", "NY100", "Trader-3", "EUROPE", 1000, -100, -100, -100 });
            dt.Rows.Add(new object[] { "ABN Amro", "Bond-JKL", "AA+", "CDS", "NY100", "Trader-1", "EUROPE", 1000, -100, -100, -100 });
            dt.Rows.Add(new object[] { "AT&T", "Bond-123", "AA+", "CDS", "NY100", "Trader-2", "NORTH AMERICA", 1000, -100, -100, -100 });
            dt.Rows.Add(new object[] { "AT&T", "Bond-456", "AA+", "CDS", "NY100", "Trader-2", "NORTH AMERICA", 1000, -100, -100, -100 });
            dt.Rows.Add(new object[] { "AT&T", "Bond-789", "A", "CDS", "NY100", "Trader-1", "EUROPE", 1000, -100, -100, -100 });
            dt.Rows.Add(new object[] { "AT&T", "Bond-XYZ", "BB", "CDS", "NY200", "Trader-1", "EUROPE", 1000, -100, -100, -100 });
            dt.Rows.Add(new object[] { "Ford", "Bond-UVW", "BB", "CDS", "NY200", "Trader-2", "NORTH AMERICA", 1000, -100, -100, -100 });
            dt.Rows.Add(new object[] { "Ford", "Bond-MNO", "BB", "CDS", "NY200", "Trader-3", "NORTH AMERICA", 1000, -100, -100, -100 });
            dt.Rows.Add(new object[] { "Ford", "Bond-PQR", "CCC", "CDS", "NY200", "Trader-3", "EUROPE", 1000, -100, -100, -100 });
            dt.Rows.Add(new object[] { "Ford", "Bond-PQR", "D", "CDS", "NY200", "Trader-3", "EUROPE", 1000, -100, -100, -100 });

            return dt;
        }    
    }
}